/**
 * We use a combination of swing and AWT components, and use an ArrayList to hold
 * references to the objects whose representations we are drawing on the screen.
 */
import java.awt.*;                                                 // Use the awt library
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;                                              // Use the swing library
import java.util.ArrayList;

/**
 * This object takes care of the graphical output and the user input through the button.
 *
 * Unlike similar objects that you have seen before, this one only deals with I/O. The 
 * creation of the objects in the simulation (and any work that the simulation makes happen)
 * takes place in the EcoSystem object.
 * 
 * written by:    Simon Parsons
 * last modified: 22nd Sepetember 2012
 */
public class SimulationComponent extends JComponent 
	implements ActionListener{
	
	JButton moveButton;                                             
	EcoSystem eSystem;
	ArrayList<Animal> animals = new ArrayList<Animal>();
		
	/**
	 * When we create the simulation component object, we create a button and set up
	 * a listener for it.
	 */
	public SimulationComponent() {
		moveButton = new JButton("Move things!");             
		setLayout(new FlowLayout());                            
		add(moveButton);
		moveButton.addActionListener(this);                        
	}
	
	/**
	 * Adding an element to the simulation puts it into the ArrayList animals
	 * 
	 */
	public void addElement(Animal animal){
		animals.add(animal);
	}

	/**
	 * Allows this object to be passed a reference to the simulation for which it
	 * is displaying objects. This allows input to be passed back.
	 */
	public void addSimulation(EcoSystem eSystem){
		this.eSystem = eSystem;
	}
	
	/** 
	 * How we paint objects
	 *
	 * Go though the list of animals, and draw a circle (an ellipse with major and 
	 * minor axis the same size) of the appropriate color.
	 * 
	 * Note:
	 * 
	 * 1) The way we use for to step through a variable sized collection.
	 * 
	 * 2) The use of instanceof to test what kind of animal we have. This is not
	 * necessarily good program practice (probably better to use an over-ridden function
	 * to identify what the color should be).
	 */
	public void paintComponent(Graphics g){               
		Graphics2D g2 = (Graphics2D)g;                   
		Shape shapes[] = new Shape[animals.size()];
		
		for(int i = 0; i < animals.size(); i++){
			shapes[i] = new Ellipse2D.Float(animals.get(i).getLocationX(), 
											animals.get(i).getLocationY(), 20, 20); 
			if(animals.get(i) instanceof Fox)
			{
				g2.setPaint(Color.orange);
			}
			else
			if(animals.get(i) instanceof Rabbit){
				if (animals.get(i).getColor() == AnimalColor.BLACK){
					g2.setPaint(Color.black);
				}
				else{
					g2.setPaint(Color.white);
				}
			}                                 
			g2.fill(shapes[i]);			
		}
	}
	
	/**
	 * What happens when we press the button
	 *
	 * Use the eSystem reference to tell the simulation to move all the animals.
	 */
	public void actionPerformed(ActionEvent e){
		
		if(e.getSource() == moveButton){
			eSystem.moveAnimals();
			repaint();		
		}
	}
}